/*
Esimerkki demoaa tietokantaan (tässä localstorage) tehtävien 
CRUD-operaatioiden päivittämistä viewiin eli HTML-templaattiin 
kahdella eri tavalla.

1) Päivitys viewiin voidaan tehdä tavalliseen tapaan, sen jälkeen
kun tietokanta (tai localstorage) on päivitetty. 

2) Päivitys viewiin voidaan tehdä optimistisesti etukäteen, 
ennen kuin tietokanta(tai localstorage) on päivitetty.

Esimerkissä tavallinen viewin päivitys tehdään tilaamalla servicen 
metodin kutsun paluuarvo subscribella komponenttiin, ja päivittämällä 
sen perusteella komponentin todos-taulukko.

Optimistinen viewin päivitys on metodeissa kommentoitu. Se tehdään
laittamalla esim. lomakkeelta saatu data heti komponentin todos-olioon,
ennen kuin on tehty muutoksia tietokantaan (localstorageen)

Voit kokeilla kommentoida tavallisen viewin päivityksen pois,
eli laittaa kommentit subscriben argumenttina olevan funktion 
ympärille. Tällöin view päivittyy vasta selaimen refreshillä,
sillä vasta silloin komponentin todos-taulukko saa uuden tilan 
localstoragesta, koska ngOnInit()-laukaistaan. 

Voit myös kokeilla poistaa optimistisen viewin päivityksen
päivityksen kommentit. Silloin komponentin todos-taulukko 
päivittyy hyvin nopeasti, mutta jos kannan käsittely jostain 
syystä epäonnistuu, on kannassa ja viewissä eri tiedot.
*/


import { Component, inject, OnInit } from '@angular/core';
import { TodoService } from './todo.service';
import { FormsModule } from '@angular/forms';
import { Todo } from './todo';

@Component({
  selector: 'app-root',
  imports: [FormsModule],
  standalone: true,
  templateUrl: './app.component.html',
  styleUrl: './app.component.css',
})
export class AppComponent implements OnInit {
  todos: Todo[] = []; // taulukko jossa ovat todot, on aluksi tyhjä
  task = '';
  id: number = 0;
  reset = ''; // tehty lisäyslomakkeen resetointia varten
  todoservice = inject(TodoService);

  ngOnInit() {
    // tyhjä taulukko localstorageen, jos siellä ei ole mitään
    this.todoservice.initStorage();
    // haetaan todot localstoragesta
    this.todoservice.getTodos().subscribe((todos) => (this.todos = todos));
  }

  // tehdään id uudelle todo-oliolle
  genId(todos: Todo[]): number {
    return todos.length > 0 ? Math.max(...todos.map((t) => t.id)) + 1 : 1;
  }

  // lisätään todo-olio localstorageen
  addTodo(input: any) {
    const genid = this.genId(this.todos);
    console.log(genid);
    const todo = { id: genid, task: input.task };
    // 2) optimistinen viewin päivitys
    //this.todos.push(todo);
    /* 1) Tavallinen viewin päivitys. Jos kommentoit subscriben 
    argumenttina olevan callbackin, uusi todo ei päivity heti 
    näytölle vaan vaaditaan reffaus */
    this.todoservice.addTodo(todo).subscribe((todo) => {
      this.todos.push(todo);
    });
    // tässä voi tyhjentää lomakkeen kentän kun lisäys on suoritettu
    this.reset = '';
  }
  // haetaan data todo-oliosta lomakkeelle päivitystä varten 
  initUpdateForm(todo: Todo){
    this.task = todo.task;
    this.id = todo.id;
  }

  // päivitetään todo-oliota
  updateTodo(input: any){
  // 2) optimistinen viewin päivitys
   /* this.todos = this.todos.map((item: Todo) => {
      if (item.id === input.id) {
        return { ...item, task: input.task };
      } else {
        return item;
      }
     }); */
    this.todoservice.updateTodo({
      'id': input.id,
      'task': input.task,
    /* 1) Tavallinen viewin päivitys. Jos kommentoit subscriben 
    argumenttina olevan callbackin, todos ei päivity heti 
    näytölle vaan vaaditaan reffaus.*/
    }).subscribe((todos) => this.todos = todos);
    // tässä voi tyhjentää lomakkeen kentän kun päivitys on suoritettu
    this.task = '';
  }
  // poistetaan todo-olio
  removeTodo(todo: Todo) {
    // 2) optimistinen viewin päivitys
    // this.todos = this.todos.filter(item => item !== todo);
    this.todoservice.removeTodo(todo)
    /* 1) Tavallinen viewin päivitys. Jos kommentoit subscriben 
    argumenttina olevan callbackin, poisto ei päivity heti 
    näytölle, vaan vaaditaan reffaus.*/
      .subscribe((todos) => this.todos = todos);
  }
}
