import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { Todo } from './todo';

@Injectable({
  providedIn: 'root',
})
export class TodoService {

  /* jos localstoragessa ei ole todos-avainta, 
     tehdään se, ja laitetaan sinne tyhjä taulukko */
  initStorage() {
    if (!localStorage.getItem('todos')) {
      localStorage.setItem('todos', JSON.stringify([]));
    }
  }

  /*
   Localstoragea käsittelevät metodit palauttavat observablen,
   joka luodaan of-metodilla. Normaalisti localstoragen käyttö ei
   vaadi asynkronista tiedonhakua observablella tai promisella, 
   mutta tässä on haluttu tehdä samalla tavalla, kuin jos meillä
   olisi palvelimella oleva tietokanta.
  */

  getTodos(): Observable<Todo[]> {
    // haetaan todot localstoragesta
    const todos = JSON.parse(localStorage.getItem('todos') || '');
    // palautetaan taulukko observablena
    return of(todos);
  }

  addTodo(todo: Todo): Observable<Todo> {
    const todos = JSON.parse(localStorage.getItem('todos') || '');
    // lisätään uusi todo-olio
    todos.push(todo);
    // lisätään localstorageen päivitetty todos
    localStorage.setItem('todos', JSON.stringify(todos));
    // lisätty todo-olio palautetaan observablena
    return of(todo);
  }

  updateTodo(todo: Todo): Observable<Todo[]> {
    const todos = JSON.parse(localStorage.getItem('todos') || '');
    /* Käydään läpi localstoragesta saatu todos-taulukko, ja
       jos alkion id on sama kuin argumenttina tulleen todon id,
       niin päivitetään kyseisen alkion task. */
    const updatedtodos = todos.map((item: Todo) => {
      if (item.id === todo.id) {
        return { ...item, task: todo.task };
      } else {
        return item;
      }
    });
    // lisätään localstorageen päivitetty todos
    localStorage.setItem('todos', JSON.stringify(updatedtodos));
    return of(updatedtodos);
  }

  removeTodo(todo: Todo): Observable<Todo[]> {
    const todos = JSON.parse(localStorage.getItem('todos') || '');
    // poistetaan todo-olio id:n perusteella
    const updatedtodos = todos.filter((item: Todo) => item.id !== todo.id);
    // lisätään localstorageen päivitetty todos
    localStorage.setItem('todos', JSON.stringify(updatedtodos));
    return of(updatedtodos);
  }
}
